/*
 * header.c -- part of ZilUtils/ZilAsm
 *
 * Copyright (C) 2016, 2019 Jason Self <j@jxself.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>
 *
 * SPDX-License-Identifier: AGPL-3.0-or-later
 */

#include <string.h>		/* bzero */
#include <assert.h>
#include <stdio.h>

#include "header.h"

Program_header_struct Program_header;

const unsigned MAX_HEADER_LEN = 40;

void
program_header_reset (unsigned zversion)
{
  bzero (&Program_header, sizeof (Program_header));
  Program_header.version = zversion;
}

ZMemblock *
program_header_build (void)
{
  ZMemblock *zmb = zmem_init (MAX_HEADER_LEN);
  zmem_putbyte (zmb, Program_header.version);
  /* TODO */
  return zmb;
}


Word
swapBytes (Word w)
{
  return ((w & 255) << 8) + (w >> 8);
}


void
outputToFile (Program_header_struct * pHeader, FILE * file)
{
  Program_header_struct h;
  memcpy (&h, pHeader, sizeof (Program_header_struct));

  h.startPC = swapBytes (h.startPC);
  h.h_file_size = swapBytes (h.h_file_size);
  h.H_STRINGS_OFFSET = swapBytes (h.H_STRINGS_OFFSET);

  int size = sizeof (Program_header_struct);
  fwrite (&h, sizeof (Program_header_struct), 1, file);
}
